#coding:utf-8
"""
Created on 30.10.14

@author: kasimova
"""
from concentrator.webservice.spyne_objects_proxy import (
    DeclaredPersonDataProxy, ApplicantDataProxy, EduOrganizationsDataProxy,
    BenefitsProxy)
from concentrator.webservice.entities import BenefitItem
from concentrator.rules import PrivilegeTypeRule
from concentrator.webservice.helpers import get_delegate_address
from concentrator.webservice.helpers import get_doc_type_delegate
from lipetsk_specifics.rules import WorkTypeRules
from lipetsk_specifics.webservice.entities import (LipetskDeclaredPersonData, LipetskApplicantData,
                      LipetskEduOrganizationsData,
                      LipetskEduOrganization,
                      WhoHaveBenefit,
                      LipetskBenefits)
from lipetsk_specifics.webservice.helpers import get_decl_priv, get_priv_data


class WhoHaveBenefitProxy(object):
    """
    Заполняем данные о владельце льготы
    """
    @staticmethod
    def create_delegate(declaration):
        conf = dict()
        priv_conf_data = get_priv_data(declaration)
        if priv_conf_data:
            conf = dict(
                Type=priv_conf_data.privilege_owner,
            )
            if priv_conf_data.delegate:
                conf.update(dict(
                    FirstName=priv_conf_data.delegate.firstname,
                    LastName=priv_conf_data.delegate.surname,
                    MiddleName=priv_conf_data.delegate.patronymic,
                    DocType=str(get_doc_type_delegate(priv_conf_data.delegate)),
                    DocSeria=priv_conf_data.delegate.dul_series,
                    DocNumber=priv_conf_data.delegate.dul_number,
                    DocIssueDate=priv_conf_data.delegate.dul_date,
                    DocIssuerName=priv_conf_data.delegate.dul_issued_by,
                    Snils=priv_conf_data.delegate.snils,
                ))

        return WhoHaveBenefit(**conf)


class LipetskBenefitsProxy(BenefitsProxy):
    """
    Отдаем только льготу пришедшую с портала,
    она должна быть одна!
    """

    @staticmethod
    def _get_privileges(declaration):
        return map(lambda declaration_privilege:
                   (declaration_privilege.privilege.id,
                    declaration_privilege.privilege.type_id,
                    declaration_privilege.doc_series,
                    declaration_privilege.doc_number),
                   get_decl_priv(declaration))

    @staticmethod
    def create_benefits(declaration):
        result = None
        privileges = LipetskBenefitsProxy._get_privileges(declaration)
        doc_info = []
        benefits = []
        for code, privilege_type, doc_series, doc_number in privileges:
            benefits.append(
                BenefitItem(
                    name=str(code),
                    Type=str(
                        PrivilegeTypeRule.get_concetr(privilege_type)))
            )
            map(lambda x: doc_info.append(x) if x else None, [doc_series,
                                                              doc_number])
        who_have_benefit = WhoHaveBenefitProxy.create_delegate(declaration)
        if benefits:
            result = LipetskBenefits(
                Benefit=benefits, BenefitsDocInfo=','.join(doc_info),
                WhoHaveBenefit=who_have_benefit)
        return result


class LipetskDeclaredPersonDataProxy(DeclaredPersonDataProxy):
    """
    Пробрасываем поле дата создания актовой записи и время пребывания в блок
    "DeclaredPerson"
    """
    DEFAULT_CLASS = LipetskDeclaredPersonData

    def _get_benefits(self):
        """
        Возвращаем данные по льготам только в том случае,
        если они присутствуют в заявке
        """
        data = dict()
        benefits = LipetskBenefitsProxy.create_benefits(self.declaration)
        if benefits:
            data['Benefits'] = LipetskBenefitsProxy.create_benefits(
                self.declaration)
        return data

    def get_extra_data(self):
        child = self.declaration.children
        work_type = WorkTypeRules.get_concetr(self.declaration.work_type_id)
        if work_type:
            work_type = str(work_type)
        data = {
            'DateOfActNumber': child.zags_act_date,
            'ArrivalTimeType': work_type,
        }
        return data


class LipetskApplicantDataProxy(ApplicantDataProxy):
    DEFAULT_CLASS = LipetskApplicantData

    def get_extra_data(self):
        child = self.declaration.children
        reg_address, fact_address = get_delegate_address(child)
        return {
            'AddressRegistration': reg_address,
            'AddressResidence': fact_address
        }


class LipetskEduOrganizationsDataProxy(EduOrganizationsDataProxy):
    DEFAULT_CLASS = LipetskEduOrganizationsData

    def _create_edu_organizations(self):
        result = []
        for du in self.declaration.declarationunit_set.all():
            mo = du.unit.get_mo()
            result.append(LipetskEduOrganization(**{
                'Code': str(du.unit.id),
                'Priority': du.ord if du.ord in [1, 2] else 2,
                'CodeMO': mo.id,
                'OkatoMO': mo.ocato,
            }))
        return result
